/********************************************************************************************************
 * @file     app.c 
 *
 * @brief    for TLSR chips
 *
 * @author	 BLE Group
 * @date     Sep. 18, 2015
 *
 * @par      Copyright (c) Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *           
 *			 The information contained herein is confidential and proprietary property of Telink 
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms 
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai) 
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in. 
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this 
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided.
 *           
 *******************************************************************************************************/
#include "tl_common.h"
#include "drivers.h"
#include "stack/ble/ble.h"
#include "battery_check.h"
#include "vendor/common/blt_common.h"
#include "../common/blt_soft_timer.h"
#include "tuya_ble_type.h"
#include "tuya_ble_api.h"
#include "ty_ble.h"
#include "ty_system.h"
#include "tuya_ble_log.h"
#include "tuya_ble_api.h"
#include "tuya_ble_sdk_test.h"
#include "ty_rtc.h"
#include "ty_uart.h"
#include "ty_pin.h"


MYFIFO_INIT(log_tx_fifo, (LOG_TXFIFO_SIZE), LOG_TXFIFO_NUM);


#define MAX_INTERVAL_VAL		16

_attribute_data_retention_	int	mcu_uart_working;
_attribute_data_retention_	int	module_uart_working;
_attribute_data_retention_	int module_task_busy;
_attribute_data_retention_	int	module_debug_log_working;

_attribute_data_retention_	int	module_uart_data_flg;
_attribute_data_retention_	u32 module_wakeup_module_tick;

extern my_fifo_t spp_rx_fifo;
extern my_fifo_t spp_tx_fifo;

#define  UART_TX_BUSY	  ((spp_tx_fifo.rptr != spp_tx_fifo.wptr) || uart_tx_is_busy() )
#define  UART_RX_BUSY	  (spp_rx_fifo.rptr != spp_rx_fifo.wptr)

#define LOG_BUSY            (log_tx_fifo.rptr != log_tx_fifo.wptr)

int app_module_busy ()
{
	//uint16_t tuya_task_status = tuya_ble_scheduler_queue_events_get();

	module_uart_working = UART_TX_BUSY || UART_RX_BUSY; //module checks to see if UART rx and tX are all processed
	module_debug_log_working = LOG_BUSY;
	module_task_busy = module_uart_working || module_debug_log_working;
	return module_task_busy;
}

void app_suspend_exit ()
{
	bls_pm_setSuspendMask(SUSPEND_DISABLE);
}

int app_suspend_enter ()
{
	if((!ty_system_is_sleep())||(app_module_busy())||(tuya_ble_sleep_allowed_check() != true)||(ty_ble_trxfifo_not_empty()))
	{
		app_suspend_exit ();
		return 0;
	}
	return 1;
}

void app_power_management ()
{
#if (BLE_MODULE_PM_ENABLE)
	if((ty_system_is_sleep())&&(!app_module_busy())&&(tuya_ble_sleep_allowed_check() == true))
	{
		#if (PM_DEEPSLEEP_RETENTION_ENABLE)
			bls_pm_setSuspendMask (SUSPEND_ADV | DEEPSLEEP_RETENTION_ADV | SUSPEND_CONN | DEEPSLEEP_RETENTION_CONN);
		#else
			bls_pm_setSuspendMask(SUSPEND_ADV | SUSPEND_CONN);
		#endif

		bls_pm_setWakeupSource(PM_WAKEUP_PAD);  // GPIO_WAKEUP_MODULE needs to be wakened
	}
#endif
}

void user_init_normal(void)
{
	//random number generator must be initiated here( in the beginning of user_init_nromal)
	//when deepSleep retention wakeUp, no need initialize again
	random_generator_init();  //this is must


	/*****************************************************************************************
	 Note: battery check must do before any flash write/erase operation, cause flash write/erase
		   under a low or unstable power supply will lead to error flash operation

		   Some module initialization may involve flash write/erase, include: OTA initialization,
				SMP initialization, ..
				So these initialization must be done after  battery check
	*****************************************************************************************/
	#if (BATT_CHECK_ENABLE)  //battery check must do before OTA relative operation
		u8 battery_check_returnVaule = 0;
		if(analog_read(USED_DEEP_ANA_REG) & LOW_BATT_FLG){
			do{
				battery_check_returnVaule = app_battery_power_check(VBAT_ALRAM_THRES_MV + 200);  //2.2 V
			}while(battery_check_returnVaule);
		}
		else{
			do{
				battery_check_returnVaule = app_battery_power_check(VBAT_ALRAM_THRES_MV);  //2.0 V
			}while(battery_check_returnVaule);
		}
	#endif


////////////////// BLE stack initialization ////////////////////////////////////
	ty_ble_init();

///////////////////// USER application initialization ///////////////////

#if (BLE_MODULE_PM_ENABLE)
	blc_ll_initPowerManagement_module();        //pm module:      	 optional
	blc_pm_setDeepsleepRetentionType(DEEPSLEEP_MODE_RET_SRAM_LOW32K);
	#if (PM_DEEPSLEEP_RETENTION_ENABLE)
		bls_pm_setSuspendMask (SUSPEND_ADV | DEEPSLEEP_RETENTION_ADV | SUSPEND_CONN | DEEPSLEEP_RETENTION_CONN);
		blc_pm_setDeepsleepRetentionThreshold(95, 95);
		blc_pm_setDeepsleepRetentionEarlyWakeupTiming(250);
	#else
		bls_pm_setSuspendMask (SUSPEND_ADV | SUSPEND_CONN);
	#endif

	bls_pm_registerFuncBeforeSuspend( &app_suspend_enter );
#else
	bls_pm_setSuspendMask (SUSPEND_DISABLE);
#endif

	blt_soft_timer_init();

	ty_system_init(0);

	tuya_ble_sdk_demo_init();

#if TUYA_BLE_SDK_TEST
	tuya_ble_sdk_test_init();
#endif

//    ty_system_wdt_init();
	ty_ble_start_adv();

}

#if (PM_DEEPSLEEP_RETENTION_ENABLE)
_attribute_ram_code_ void user_init_deepRetn(void)
{
	blc_ll_initBasicMCU();   //mandatory
	ty_ble_set_tx_power(RF_POWER_P3p01dBm);

	blc_ll_recoverDeepRetention();

	extern  int sdk_mainLoop_run_flag;
	sdk_mainLoop_run_flag = 0;
	extern  u8 blt_dma_tx_rptr;
	blt_dma_tx_rptr = 0;

	irq_enable();

	////////////////// SPP initialization ///////////////////////////////////
	//note: dma addr must be set first before any other uart initialization!
	ty_uart_init();
	
	ty_pin_init(0,0);

    blt_soft_timer_init();
}
#endif

void  log_exe(void)
{
	u16 len = 0,i = 0;
	u8 log_data[256];
	u8 *p = my_fifo_get (&log_tx_fifo);
	if (p)
	{
		memcpy(log_data, p + 2, p[0]+p[1]*256);
		len = p[0]+p[1]*256 ;
		while(len)
		{
			putchar(log_data[i++]);
			len--;
		}
		my_fifo_pop (&log_tx_fifo);
	}
	return;
}

extern int rx_from_uart_cb (void);
extern int tx_to_uart_cb (void);


void main_loop (void)
{
    log_exe();

    ty_timer_process();

	blt_sdk_main_loop();

	ty_system_mainloop();

	blt_soft_timer_process(MAINLOOP_ENTRY);

	rx_from_uart_cb();
	tx_to_uart_cb();

	app_power_management ();

}
