/********************************************************************************************************
 * @file     app_adc.c 
 *
 * @brief    for TLSR chips
 *
 * @author	 public@telink-semi.com;
 * @date     May. 12, 2018
 *
 * @par      Copyright (c) Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *           
 *			 The information contained herein is confidential and proprietary property of Telink 
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms 
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai) 
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in. 
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this 
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided. 
 *           
 *******************************************************************************************************/
#include "ty_adc.h"
#include "tl_common.h"
#include "drivers.h"

void adc_gpio_ain_init(ty_adc_t* p_adc)
{
    ////Step 1: power off sar adc/////////////////////////////////////////////////////////
	/******power off sar adc********/
	adc_power_on_sar_adc(0);
	//////////////////////////////////////////////////////////////////////////////////////

	////Step 2: Config some common adc settings(user can not change these)/////////////////
	/******enable signal of 24M clock to sar adc********/
	adc_enable_clk_24m_to_sar_adc(1);

	/******set adc sample clk as 4MHz******/
	adc_set_sample_clk(5); //adc sample clk= 24M/(1+5)=4M

	/******set adc L R channel Gain Stage bias current trimming******/
	adc_set_left_gain_bias(GAIN_STAGE_BIAS_PER100);
	adc_set_right_gain_bias(GAIN_STAGE_BIAS_PER100);
	////////////////////////////////////////////////////////////////////////////////////////

	////Step 3: Config adc settings  as needed /////////////////////////////////////////////	
	//set misc channel en,  and adc state machine state cnt 2( "set" stage and "capture" state for misc channel)
	adc_set_chn_enable_and_max_state_cnt(ADC_MISC_CHN, 2);  	//set total length for sampling state machine and channel

	//set "capture state" length for misc channel: 240
	//set "set state" length for misc channel: 10
	//adc state machine  period  = 24M/250 = 96K, T = 10.4 uS
	adc_set_state_length(240, 0, 10);  	//set R_max_mc,R_max_c,R_max_s


	//set misc channel use differential_mode (telink advice: only differential mode is available)
	//single mode adc source, PB4 for example: PB4 positive channel, GND negative channel
	gpio_set_func(p_adc->pin, AS_GPIO);
	gpio_set_input_en(p_adc->pin, 0);
	gpio_set_output_en(p_adc->pin, 0);
	gpio_write(p_adc->pin, 0);
	adc_set_ain_channel_differential_mode(ADC_MISC_CHN, p_adc->channel, GND);

	//set misc channel resolution 14 bit
	//notice that: in differential_mode MSB is sign bit, rest are data,  here BIT(13) is sign bit
	adc_set_resolution(ADC_MISC_CHN, RES14);  //set resolution

	//set misc channel vref 1.2V
	adc_set_ref_voltage(ADC_MISC_CHN, ADC_VREF_1P2V);  					//set channel Vref

	//set misc t_sample 6 cycle of adc clock:  6 * 1/4M
	adc_set_tsample_cycle(ADC_MISC_CHN, SAMPLING_CYCLES_6);  	//Number of ADC clock cycles in sampling phase

	//set Analog input pre-scaling 1/8
	adc_set_ain_pre_scaler(ADC_PRESCALER_1F8);
}


uint32_t ty_adc_init(ty_adc_t* p_adc)
{
	adc_gpio_ain_init(p_adc);

////Step 4: power on sar adc/////////////////////////////////////////////////////////
		/******power on sar adc********/
	adc_power_on_sar_adc(1);
////////////////////////////////////////////////////////////////////////////////////////
	return 0;
}

uint32_t ty_adc_start(ty_adc_t* p_adc)
{
	ty_adc_init(p_adc);

	u32 Adc_cur_rawData;   //unit: m V
	Adc_cur_rawData =  adc_sample_and_get_result();
	p_adc->value = (uint16_t)Adc_cur_rawData;
	return Adc_cur_rawData;
}

uint32_t ty_adc_stop(ty_adc_t* p_adc)
{
	return 0;
}

uint32_t ty_adc_control(ty_adc_t* p_adc, uint8_t cmd, void* arg)
{
	return 0;
}

uint32_t ty_adc_uninit(ty_adc_t* p_adc)
{
	adc_power_on_sar_adc(0);
	// in order to reduce consumption ,user may need config the io state of adc(input,pull high or down or float)
	return 0;
}
