#include "drivers.h"
#include "stack/ble/ble.h"
#include "stack/ble/gap/gap_event.h"
#include "vendor/common/blt_common.h"
#include "ty_ble.h"
#include "tuya_ble_api.h"
#include "tuya_ble_ota.h"
#include "tuya_ble_utils.h"
#include "tuya_ble_log.h"

/*********************************************************************
 * LOCAL CONSTANT
 */
#define     DEFAULT_ADV_DATA    \
            {                   \
                3,              \
                {               \
                    0x02,       \
                    0x01,       \
                    0x06,       \
                },              \
            }
            
#define     DEFAULT_SCAN_RSP    \
            {                   \
                13,              \
                {               \
                    0x0C,       \
                    0x09,       \
                    't', 'y', '_', 'b', 'l', 'e', '_', 'd', 'e', 'm', 'o', \
                },              \
            }

#define     DEFAULT_ADV_PARAM               \
            {                               \
                .adv_interval_min = TY_ADV_INTERVAL,    \
                .adv_interval_max = TY_ADV_INTERVAL,    \
                .adv_type         = 0x01,   \
                .adv_power        = 0x00,   \
                .adv_channal_map  = 0x07,   \
            }

#define     TY_BLE_CONN_CFG_TAG      (1)



//#define 	MY_ADV_INTERVAL_MIN				ADV_INTERVAL_1S
//#define 	MY_ADV_INTERVAL_MAX				ADV_INTERVAL_505MS+ADV_INTERVAL_500MS

#define		TY_RF_POWER_INDEX				RF_POWER_P3p01dBm

/*********************************************************************
 * LOCAL STRUCT
 */

/*********************************************************************
 * LOCAL VARIABLE
 */
static ty_ble_data_t        s_adv_data  = DEFAULT_ADV_DATA;
static ty_ble_data_t        s_scan_rsp  = DEFAULT_SCAN_RSP;
static ty_ble_adv_param_t   s_adv_param = DEFAULT_ADV_PARAM;


u8  mac_public[6];
u8  mac_type;

#define		BLE_DEVICE_ADDRESS_TYPE 		BLE_DEVICE_ADDRESS_RANDOM_STATIC
_attribute_data_retention_	own_addr_type_t 	app_own_address_type = OWN_ADDRESS_RANDOM;

#define TUYA_BLE_DATA_LENGTH		MAX_OCTETS_DATA_LEN_EXTENSION//MAX_OCTETS_DATA_LEN_27//MAX_OCTETS_DATA_LEN_EXTENSION

#define RX_FIFO_SIZE	((TUYA_BLE_DATA_LENGTH+24)/16+1)*16	//rx-24   max:251+24 = 275  16 align-> 288
#define RX_FIFO_NUM		16

#define TX_FIFO_SIZE	((TUYA_BLE_DATA_LENGTH+12)/4+1)*4	//tx-12   max:251+12 = 263  4 align-> 264
#define TX_FIFO_NUM		16  //if TUYA_BLE_DATA_LENGTH is 27,need to be set 16

_attribute_no_ret_bss_ u8 		 	blt_rxfifo_b[RX_FIFO_SIZE * RX_FIFO_NUM] = {0};
_attribute_no_ret_data_	my_fifo_t	blt_rxfifo = {
												RX_FIFO_SIZE,
												RX_FIFO_NUM,
												0,
												0,
												blt_rxfifo_b,};


_attribute_no_ret_bss_ u8 		 	blt_txfifo_b[TX_FIFO_SIZE * TX_FIFO_NUM] = {0};
_attribute_no_ret_data_	my_fifo_t	blt_txfifo = {
												TX_FIFO_SIZE,
												TX_FIFO_NUM,
												0,
												0,
												blt_txfifo_b,};

#define 	MTU_RX_MAX						TUYA_BLE_DATA_MTU_MAX+3

#define		MTU_RX_BUFF_SIZE_MAX			ATT_ALLIGN4_DMA_BUFF(MTU_RX_MAX)
#define		MTU_TX_BUFF_SIZE_MAX			ATT_ALLIGN4_DMA_BUFF(MTU_RX_MAX)

_attribute_data_retention_ u8 mtu_rx_fifo[MTU_RX_BUFF_SIZE_MAX];
_attribute_data_retention_ u8 mtu_tx_fifo[MTU_TX_BUFF_SIZE_MAX];
//adv
static bool is_advertising = false;

/*********************************************************************
 * VARIABLE
 */

/*********************************************************************
 * LOCAL FUNCTION
 */

/*********************************************************
FN:
*/
extern tuya_ble_parameters_settings_t tuya_ble_current_para;
static void ty_ble_mac_init(void)
{
	//u8  mac_public[6];
	u8  mac_random_static[6];
	//for 512K Flash, flash_sector_mac_address equals to 0x76000
	//for 1M   Flash, flash_sector_mac_address equals to 0xFF000
	//blc_initMacAddress(flash_sector_mac_address, mac_public, mac_random_static);

	tuya_ble_storage_load_settings();

	if(tuya_ble_license_key_type())
	{
		u8 mac_temp[6];
		tuya_ble_str_to_hex(TY_DEVICE_MAC,MAC_STRING_LEN, mac_temp);
		for(u8 i=0;i<6;i++)
		{
			mac_public[i] = mac_temp[5-i];
		}
	}
	else
	{
		memcpy(mac_public, tuya_ble_current_para.auth_settings.mac, MAC_LEN);
	}

	TUYA_APP_LOG_INFO("MAC addr init");
	TUYA_APP_LOG_HEXDUMP_DEBUG("mac init:", mac_public, MAC_LEN);

#if(BLE_DEVICE_ADDRESS_TYPE == BLE_DEVICE_ADDRESS_PUBLIC)
	app_own_address_type = OWN_ADDRESS_PUBLIC;
#elif(BLE_DEVICE_ADDRESS_TYPE == BLE_DEVICE_ADDRESS_RANDOM_STATIC)
	app_own_address_type = OWN_ADDRESS_RANDOM;
	blc_ll_setRandomAddr(mac_public);
#endif

	mac_type = app_own_address_type;

////// Controller Initialization  //////////
	blc_ll_initBasicMCU();                      //mandatory
	blc_ll_initStandby_module(mac_public);				//mandatory
	blc_ll_initAdvertising_module(mac_public); 	//adv module: 		 mandatory for BLE slave,
	blc_ll_initConnection_module();				//connection module  mandatory for BLE slave/master
	blc_ll_initSlaveRole_module();				//slave module: 	 mandatory for BLE slave,
}

static int app_conn_param_update_response(u8 id, u16  result)
{
#if 0
	if(result == CONN_PARAM_UPDATE_ACCEPT){
		printf("SIG: the LE master Host has accepted the connection parameters.\n");
		conn_update_cnt = 0;
	}
	else if(result == CONN_PARAM_UPDATE_REJECT)
	{
		printf("SIG: the LE master Host has rejected the connection parameters..\n");
		printf("Current Connection interval:%dus.\n", bls_ll_getConnectionInterval() * 1250 );
		conn_update_cnt++;
		if(conn_update_cnt < 4){
			printf("Slave sent update connPara req!\n");
		}
		if(conn_update_cnt == 1){
			bls_l2cap_requestConnParamUpdate (8, 16, 0, 400);//18.75ms iOS
		}
		else if(conn_update_cnt == 2){
			bls_l2cap_requestConnParamUpdate (16,32, 0, 400);
		}
		else if(conn_update_cnt == 3){
			bls_l2cap_requestConnParamUpdate (32,60, 0, 400);
		}
		else{
			conn_update_cnt = 0;
			printf("Slave Connection Parameters Update table all tested and failed!\n");
		}
	}
#endif

	return 0;
}

static int app_host_event_callback (u32 h, u8 *para, int n)
{
	u8 event = h & 0xFF;

	switch(event)
	{
		case GAP_EVT_SMP_PARING_BEAGIN:
		{

		}
		break;

		case GAP_EVT_SMP_PARING_SUCCESS:
		{
			gap_smp_paringSuccessEvt_t* p = (gap_smp_paringSuccessEvt_t*)para;

			if(p->bonding_result){

			}
			else{

			}
		}
		break;

		case GAP_EVT_SMP_PARING_FAIL:
		{
			gap_smp_paringFailEvt_t* p = (gap_smp_paringFailEvt_t*)para;
		}
		break;

		case GAP_EVT_SMP_CONN_ENCRYPTION_DONE:
		{
			gap_smp_connEncDoneEvt_t* p = (gap_smp_connEncDoneEvt_t*)para;

			if(p->re_connect == SMP_STANDARD_PAIR){  //first paring

			}
			else if(p->re_connect == SMP_FAST_CONNECT){  //auto connect

			}
		}
		break;

		case GAP_EVT_SMP_TK_DISPALY:
		{
			char pc[7];
			u32 pinCode = *(u32*)para;
		}
		break;

		case GAP_EVT_SMP_TK_REQUEST_PASSKEY:
		{

		}
		break;

		case GAP_EVT_SMP_TK_REQUEST_OOB:
		{

		}
		break;

		case GAP_EVT_SMP_TK_NUMERIC_COMPARE:
		{
			char pc[7];
			u32 pinCode = *(u32*)para;
		}
		break;
		case GAP_EVT_ATT_EXCHANGE_MTU:
		{
			gap_gatt_mtuSizeExchangeEvt_t *p = (gap_gatt_mtuSizeExchangeEvt_t*)para;
			TUYA_APP_LOG_INFO("host mtu: %d", p->effective_MTU);
		}
		break;
		default:
		break;
	}

	return 0;
}


static void ty_ble_host_init(void)
{
////// Host Initialization  //////////
	blc_gap_peripheral_init();    //gap initialization
	extern void my_att_init ();
	my_att_init (); //gatt initialization
	blc_l2cap_register_handler (blc_l2cap_packet_receive);  	//l2cap initialization
	blc_l2cap_registerConnUpdateRspCb(app_conn_param_update_response);         //register sig process handler
	blc_l2cap_initMtuBuffer(mtu_rx_fifo, MTU_RX_BUFF_SIZE_MAX, mtu_tx_fifo, MTU_TX_BUFF_SIZE_MAX);

	blc_att_setRxMtuSize(MTU_RX_MAX);

	// To Recive host mtu 
	blc_gap_setEventMask(GAP_EVT_MASK_ATT_EXCHANGE_MTU);
	blc_gap_registerHostEventHandler(app_host_event_callback);
}

static void ty_ble_smp_init(void)
{
	//Smp Initialization may involve flash write/erase(when one sector stores too much information,
	//   is about to exceed the sector threshold, this sector must be erased, and all useful information
	//   should re_stored) , so it must be done after battery check
	#if (BLE_SECURITY_ENABLE)
		//defalut smp4.0, just work
		blc_smp_peripheral_init();

		#if (0) //default close
			//host(GAP/SMP/GATT/ATT) event process: register host event callback and set event mask
			blc_gap_registerHostEventHandler( app_host_event_callback );
			blc_gap_setEventMask( GAP_EVT_MASK_SMP_PARING_BEAGIN 			|  \
								  GAP_EVT_MASK_SMP_PARING_SUCCESS   		|  \
								  GAP_EVT_MASK_SMP_PARING_FAIL				|  \
								  GAP_EVT_MASK_SMP_CONN_ENCRYPTION_DONE );
		#endif
	#else
		blc_smp_setSecurityLevel(No_Security);
	#endif
}

static void ty_ble_adv_init(void)
{
	bls_ll_setAdvData( s_adv_data.value, s_adv_data.len);
	bls_ll_setScanRspData( s_scan_rsp.value, s_scan_rsp.len);



	////////////////// config adv packet /////////////////////
	u8 status = bls_ll_setAdvParam( (s_adv_param.adv_interval_min*8)/5, (s_adv_param.adv_interval_max*8)/5,
								 ADV_TYPE_CONNECTABLE_UNDIRECTED, app_own_address_type,
								 0,  NULL,
								 BLT_ENABLE_ADV_ALL,
								 ADV_FP_NONE);

	if(status != BLE_SUCCESS) { 	while(1); }  //debug: adv setting err
}

extern void app_suspend_exit ();
static int controller_event_handler(u32 h, u8 *para, int n)
{

	if((h&HCI_FLAG_EVENT_TLK_MODULE)!= 0)			//module event
	{

		u8 event = (u8)(h&0xff);

		switch(event)
		{
			case BLT_EV_FLAG_SCAN_RSP:
			break;


			case BLT_EV_FLAG_CONNECT:
			{
				TUYA_BLE_LOG_INFO("BLT_EV_FLAG_CONNECT:Connected");
				tuya_ble_connected_handler();
				bls_l2cap_requestConnParamUpdate ((TY_CONN_INTERVAL_MIN*4)/5, (TY_CONN_INTERVAL_MAX*4)/5, 0, 600);
				//MTU size reset to default 23 bytes every new connection, it can be only updated by MTU size exchange procedure
				blc_att_requestMtuSizeExchange(BLS_CONN_HANDLE, MTU_RX_MAX);

				blc_ll_exchangeDataLength(LL_LENGTH_REQ , TUYA_BLE_DATA_LENGTH);
			}
			break;


			case BLT_EV_FLAG_TERMINATE:
			{
				tuya_ble_disconnected_handler();
				tuya_ble_ota_disconn_handler();
			}
			break;

			case BLT_EV_FLAG_GPIO_EARLY_WAKEUP:
			break;

			case BLT_EV_FLAG_CHN_MAP_REQ:
			break;


			case BLT_EV_FLAG_CONN_PARA_REQ:
			{
				//Slave received Master's LL_Connect_Update_Req pkt.
				rf_packet_ll_updateConnPara_t p;
				memcpy((u8*)&p.winSize, para, 11);
				TUYA_BLE_LOG_INFO("Receive Master's LL_Connect_Update_Req pkt.\n");
//				printf("Receive Master's LL_Connect_Update_Req pkt.\n");
//				printf("Connection interval:%dus.\n", p.interval*1250);
			}
			break;


			case BLT_EV_FLAG_CHN_MAP_UPDATE:
			{
			}
			break;


			case BLT_EV_FLAG_CONN_PARA_UPDATE:
			{
				//Master send SIG_Connection_Param_Update_Rsp pkt,and the reply result is 0x0000. When connection event counter value is equal
				//to the instant, a callback event BLT_EV_FLAG_CONN_PARA_UPDATE will generate. The connection interval at this time should be the
				//currently updated and valid connection interval!
				TUYA_BLE_LOG_INFO("Update param event occur.\n");
//				printf("Current Connection interval:%dus.\n", bls_ll_getConnectionInterval() * 1250);
			}
			break;


			case BLT_EV_FLAG_ADV_DURATION_TIMEOUT:
			break;


			case BLT_EV_FLAG_SUSPEND_ENTER:
			break;


			case BLT_EV_FLAG_SUSPEND_EXIT:
				app_suspend_exit ();
			break;


			default:
			break;
		}
	}
}

void ty_ble_evt_init(void)
{
	blc_hci_registerControllerEventHandler(controller_event_handler);		//register event callback
	bls_hci_mod_setEventMask_cmd(0xfffff);			//enable all 18 events,event list see ll.h
}
/*********************************************************
FN: 
*/
uint32_t ty_ble_init(void)
{
////////////////////// BLE stack initialization //////////////////
	ty_ble_mac_init();
	ty_ble_host_init();
	ty_ble_smp_init();

///////////////////// USER application initialization /////////////
	ty_ble_adv_init();
	ty_ble_set_tx_power(TY_RF_POWER_INDEX);
	ty_ble_evt_init();

    return TUYA_BLE_SUCCESS;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_start_adv(void)
{
    uint32_t ret = bls_ll_setAdvEnable(1);  //adv enable
    is_advertising = true;
    return ret;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_restart_adv(void)
{
	uint32_t ret = bls_ll_setAdvEnable(1);  //adv enable
    is_advertising = true;
    return ret;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_stop_adv(void)
{
	uint32_t ret = bls_ll_setAdvEnable(0);  //adv disable
    is_advertising = false;
    return ret;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_set_advdata_and_scanrsp(const ty_ble_data_t* p_adv_data, const ty_ble_data_t* p_scan_rsp)
{
    bool adv_flag = false;
    uint32_t ret = 0;

    if(is_advertising) {
        adv_flag = true;
        ty_ble_stop_adv();
    }

    if(p_adv_data != NULL) {
    	ret = bls_ll_setAdvData(p_adv_data->value,p_adv_data->len);
    }
    if(p_scan_rsp != NULL) {
    	ret = bls_ll_setScanRspData(p_scan_rsp->value,p_scan_rsp->len);
    }

    if(adv_flag) {
        ty_ble_start_adv();
    }
    return ret;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_set_adv_param(const ty_ble_adv_param_t* p_param)
{
    bool adv_flag = false;

    if(is_advertising) {
        adv_flag = true;
        ty_ble_stop_adv();
    }

    ////////////////// config adv packet /////////////////////
	u8 status = bls_ll_setAdvParam( (p_param->adv_interval_min*8)/5, (p_param->adv_interval_max*8)/5,
								 ADV_TYPE_CONNECTABLE_UNDIRECTED, app_own_address_type,
								 0,  NULL,
								 BLT_ENABLE_ADV_ALL,
								 ADV_FP_NONE);

	if(status != BLE_SUCCESS) { 	while(1); }  //debug: adv setting err

    if(adv_flag) {
        ty_ble_start_adv();
    }
    return TUYA_BLE_SUCCESS;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_connect(const ty_ble_mac_t* p_mac)
{
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_reconnect(const ty_ble_mac_t* p_mac)
{
    return TUYA_BLE_SUCCESS;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_disconnect(void)
{
	uint32_t ret = 0;
    if(blc_ll_getCurrentState()==BLS_LINK_STATE_CONN)
	{
    	ret = bls_ll_terminateConnection(HCI_ERR_REMOTE_USER_TERM_CONN);
	}
    return ret;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_set_conn_param(uint16_t cMin, uint16_t cMax, uint16_t latency, uint16_t timeout)
{
	bls_l2cap_requestConnParamUpdate ((cMin*4)/5, (cMax*4)/5, latency, timeout/10);

	return TUYA_BLE_SUCCESS;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_set_mac(const ty_ble_mac_t* p_mac)
{
    bool adv_flag = false;

    if(is_advertising) {
        adv_flag = true;
        ty_ble_stop_adv();
    }
    
    memcpy(mac_public,p_mac->addr,6);
	mac_type = p_mac->type;

	extern rf_packet_scan_rsp_t	pkt_scan_rsp;
	if(mac_type == 0)
	{
		memcpy((char *)bltMac.macAddress_public, (char *)mac_public, 6);
		memcpy((char *)pkt_adv.advA, (char *)mac_public, 6);
		memcpy((char *)pkt_scan_rsp.advA, (char *)mac_public, 6);
	}

    if(adv_flag) {
        ty_ble_start_adv();
    }
    return TUYA_BLE_SUCCESS;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_get_mac(ty_ble_mac_t* p_mac)
{
    if(mac_type == OWN_ADDRESS_PUBLIC)
    {
        p_mac->type = TUYA_BLE_ADDRESS_TYPE_PUBLIC;
    }
    else
    {
        p_mac->type = TUYA_BLE_ADDRESS_TYPE_RANDOM;
    }
    
    memcpy(p_mac->addr,mac_public,6);

    return TUYA_BLE_SUCCESS;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_connect_handler(void)
{
    is_advertising = false;
    tuya_ble_connected_handler();
    return TUYA_BLE_SUCCESS;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_disconnect_handler(void)
{
    tuya_ble_disconnected_handler();
    tuya_ble_ota_disconn_handler();
    return TUYA_BLE_SUCCESS;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_receive_data_handler(const uint8_t* buf, uint32_t size)
{
//    TY_PRINTF("ble rx size: %d", size);
	uint32_t ret = tuya_ble_gatt_receive_data((void*)buf, size);
    return ret;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_send_data(const uint8_t* buf, uint32_t size)
{
    uint16_t data_len = size;
    if(data_len > TUYA_BLE_DATA_MTU_MAX)
    {
        data_len = TUYA_BLE_DATA_MTU_MAX;
    }
    
    if(bls_ll_isConnectState())
	{
		return bls_att_pushNotifyData(0x11, buf, data_len);
	}
	else
	{
		return TUYA_BLE_ERR_INVALID_STATE;
	}
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_get_rssi(int8_t* p_rssi)
{
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_set_tx_power(int8_t tx_power)
{
	rf_set_power_level_index (tx_power);
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_set_device_name(const uint8_t* buf, uint16_t size)
{
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_get_device_name(uint8_t* buf, uint16_t* p_size)
{
}

/*********************************************************
FN: 
*/
uint32_t ty_ble_set_dle(void)
{
	bls_pm_setSuspendMask (SUSPEND_DISABLE);
}


/*********************************************************
FN:
*/
uint32_t ty_ble_trxfifo_not_empty(void)
{
	return ((blt_rxfifo.rptr!=blt_rxfifo.wptr)||(blt_txfifo.rptr!=blt_txfifo.wptr));
}



