/********************************************************************************************************
 * @file     app_pwm.c 
 *
 * @brief    for TLSR chips
 *
 * @author	 public@telink-semi.com;
 * @date     May. 12, 2018
 *
 * @par      Copyright (c) Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *           
 *			 The information contained herein is confidential and proprietary property of Telink 
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms 
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai) 
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in. 
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this 
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided. 
 */
#include "ty_pwm.h"
#include "tl_common.h"
#include "drivers.h"

/*********************************************************************************
    PWM0   :  PA2.  PC1. PD5
    PWM1   :  PA3.  PC3.
    PWM2   :  PA4.  PC4.
    PWM3   :  PB0.  PD2.
    PWM4   :  PB1.  PB4.
    PWM5   :  PB2.  PB5.
    PWM0_N :  PA0.  PB3.  PC4	PD5
    PWM1_N :  PC1.  PD3.
    PWM2_N :  PD4.
    PWM3_N :  PC5.
    PWM4_N :  PC0.  PC6.
    PWM5_N :  PC7.
 *********************************************************************************/

uint32_t ty_pwm_init(ty_pwm_t* p_pwm)
{
    uint16_t duty = 0;
    float ration = 0.0;
    if(p_pwm == NULL){
        return 1;
    }

    if(p_pwm->duty == 0)
    {
        return 2;
    }

    if(p_pwm->id < PWM0_ID || p_pwm->id > PWM5_ID)
    {
        return 3;
    }

    
    ration = ((float)p_pwm->freq) / ((float)100.0 / p_pwm->duty);
    duty = (uint16_t)ration;

    pwm_set_clk(CLOCK_SYS_CLOCK_HZ, CLOCK_SYS_CLOCK_HZ);
	gpio_set_func(p_pwm->pin, AS_PWM4);
	pwm_set_mode(p_pwm->id, PWM_NORMAL_MODE);
	pwm_set_phase(p_pwm->id, 0);   //no phase at pwm beginning
	pwm_set_cycle_and_duty(p_pwm->id, (u16) (p_pwm->freq * CLOCK_SYS_CLOCK_1US),  (u16) (duty * CLOCK_SYS_CLOCK_1US) );

    return 0;
}

uint32_t ty_pwm_start(ty_pwm_t* p_pwm)
{
    uint16_t duty = 0;
    float ration = 0.0;
    if(p_pwm == NULL)
    {
        return 1;
    }

    if(p_pwm->duty == 0)
    {
        return 2;
    }

    if(p_pwm->id < PWM0_ID || p_pwm->id > PWM5_ID)
    {
        return 3;
    }
    
    ration = ((float)p_pwm->freq) / ((float)100.0 / p_pwm->duty);
    duty = (uint16_t)ration;   
	pwm_set_cycle_and_duty(p_pwm->id, (u16) (p_pwm->freq * CLOCK_SYS_CLOCK_1US),  (u16) (duty * CLOCK_SYS_CLOCK_1US) );

	pwm_start(p_pwm->id);
    return 0;    
}

uint32_t ty_pwm_stop(ty_pwm_t* p_pwm)
{
    if(p_pwm == NULL){
        return 1;
    }

    if(p_pwm->id < PWM0_ID || p_pwm->id > PWM5_ID)
    {
        return 3;
    }

	pwm_stop(p_pwm->id);
    return 0;
}

uint32_t ty_pwm_control (ty_pwm_t* p_pwm, uint8_t cmd, void* arg)
{
    return 0;
}

uint32_t ty_pwm_uninit  (ty_pwm_t* p_pwm)
{
    ty_pwm_stop(p_pwm);
    return 0;
}

