#include "ty_system.h"
#include "ty_ble.h"
#include "ty_uart.h"
#include "ty_flash.h"
#include "ty_timer.h"
#include "ty_rtc.h"
#include "ty_pwm.h"
#include "ty_adc.h"
#include "ty_pin.h"
#include "ty_i2c.h"
#include "ty_spi.h"
#include "tuya_ble_api.h"
#include "tuya_ble_log.h"
#include "tuya_ble_sdk_test.h"
#include "stack/ble/ble.h"
#include "drivers/8258/watchdog.h"
#include "vendor/8258_module/app_config.h"

/*********************************************************************
 * LOCAL CONSTANT
 */
            
/*********************************************************************
 * LOCAL STRUCT
 */

/*********************************************************************
 * LOCAL VARIABLE
 */
_attribute_data_retention_ volatile bool g_system_sleep = false;

/*********************************************************************
 * VARIABLE
 */

/*********************************************************************
 * LOCAL FUNCTION
 */




/*********************************************************
FN: 
*/
uint32_t ty_system_init(uint8_t location)
{
    switch(location)
    {
        case 0: {
            ty_system_log_init();
            //peripheral init
            ty_flash_init();
            ty_uart_init();
            ty_rtc_init();
        } break;
        
        case 1: {
        } break;
        
        case 2: {
        } break;
        
        default: {
        } break;
    }
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_mainloop(void)
{
    tuya_ble_main_tasks_exec();
    return 0;
}

static tuya_ble_timer_t  tuya_ble_reboot ;
void tuya_ble_reboot_callback(tuya_ble_timer_t timer)
{
	tuya_ble_gap_disconnect();
    irq_disable();
	start_reboot();
}

/*********************************************************
FN:
*/
uint32_t ty_system_reset(void)
{
	tuya_ble_timer_create(&tuya_ble_reboot,1000, TUYA_BLE_TIMER_SINGLE_SHOT,tuya_ble_reboot_callback);
	tuya_ble_timer_start(tuya_ble_reboot);
    return 0;
}

/*********************************************************
FN: 
*/
static void wdt_event_handler(void)
{
    //NOTE: The max amount of time we can spend in WDT interrupt is two cycles of 32768[Hz] clock - after that, reset occurs
}
uint32_t ty_system_wdt_init(void)
{
#if (MODULE_WATCHDOG_ENABLE)
	wd_stop();
	wd_clear();
	wd_start();
#endif
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_wdt_uninit(void)
{
#if (MODULE_WATCHDOG_ENABLE)
	wd_stop();
#endif
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_wdt_feed(void)
{
#if (MODULE_WATCHDOG_ENABLE)
	wd_clear();
#endif
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_log_init(void)
{
#if (TY_LOG_ENABLE||TUYA_BLE_LOG_ENABLE||TUYA_APP_LOG_ENABLE)
    /* initialize EasyLogger */
    elog_init();
    /* set EasyLogger log format */
    elog_set_fmt(ELOG_LVL_ASSERT,  ELOG_FMT_LVL | ELOG_FMT_TAG | ELOG_FMT_TIME|ELOG_FMT_FUNC|ELOG_FMT_LINE);
    elog_set_fmt(ELOG_LVL_ERROR, ELOG_FMT_LVL | ELOG_FMT_TAG | ELOG_FMT_TIME|ELOG_FMT_FUNC|ELOG_FMT_LINE);
    elog_set_fmt(ELOG_LVL_WARN, ELOG_FMT_LVL | ELOG_FMT_TAG | ELOG_FMT_TIME|ELOG_FMT_FUNC|ELOG_FMT_LINE);
    elog_set_fmt(ELOG_LVL_INFO, ELOG_FMT_LVL | ELOG_FMT_TAG | ELOG_FMT_TIME);
    elog_set_fmt(ELOG_LVL_DEBUG, ELOG_FMT_TIME);
    //elog_set_fmt(ELOG_LVL_DEBUG, ELOG_FMT_LVL);
    //elog_set_fmt(ELOG_LVL_VERBOSE, ELOG_FMT_LVL);
    /* start EasyLogger */
    elog_start();
#endif
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_log_hexdump(const char *name, uint8_t width, uint8_t *buf, uint16_t size)
{
#if (TY_LOG_ENABLE||TUYA_BLE_LOG_ENABLE||TUYA_APP_LOG_ENABLE)
    elog_hexdump(name, width, buf, size);
#endif
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_delay_ms(uint32_t ms)
{
    sleep_ms(ms);
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_delay_us(uint32_t us)
{
    sleep_us(us);
    return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_enter_sleep(void)
{
	g_system_sleep = true;
	cpu_sleep_wakeup(DEEPSLEEP_MODE_RET_SRAM_LOW32K, PM_WAKEUP_PAD, 0);
    return 0;
}

/*********************************************************
FN:
*/
uint32_t ty_system_exit_sleep(void)
{
	g_system_sleep = false;
    return 0;
}

/*********************************************************
FN:
*/
uint32_t ty_system_enter_suspend(void)
{
	g_system_sleep = true;
	//bls_pm_setSuspendMask (SUSPEND_ADV | DEEPSLEEP_RETENTION_ADV | SUSPEND_CONN | DEEPSLEEP_RETENTION_CONN);
	return 0;
}

/*********************************************************
FN: 
*/
uint32_t ty_system_exit_suspend(void)
{
	g_system_sleep = false;
	return 0;
}
/*********************************************************
FN: 
*/
bool ty_system_is_sleep(void)
{
    return g_system_sleep;
}






