#include "ty_uart.h"
#include "tl_common.h"
#include "drivers.h"
#include "stack/ble/ble.h"
#include "vendor/common/blt_common.h"
#include "tuya_ble_log.h"
#include "tuya_ble_api.h"

#define  TLSR_UART_GPIO_TX       UART_TX_PB1
#define  TLSR_UART_GPIO_RX       UART_RX_PB7

#define  TLSR_UART_BAUD_9600_P0       118
#define  TLSR_UART_BAUD_9600_P1       13

my_fifo_t spp_rx_fifo;
my_fifo_t spp_tx_fifo;

//#define  UART_TX_BUSY	  ((spp_tx_fifo.rptr != spp_tx_fifo.wptr) || uart_tx_is_busy() )
//#define  UART_RX_BUSY	  (spp_rx_fifo.rptr != spp_rx_fifo.wptr)


MYFIFO_INIT(spp_rx_fifo, (SPP_RXFIFO_SIZE), SPP_RXFIFO_NUM);
MYFIFO_INIT(spp_tx_fifo, (SPP_TXFIFO_SIZE), SPP_TXFIFO_NUM);

/*********************************************************
FN:
*/
/////////////////////////////////////blc_register_hci_handler for spp////////////////////////////
int rx_from_uart_cb (void)//UART data send to Master,we will handler the data as CMD or DATA
{
	u8* p = my_fifo_get(&spp_rx_fifo);
	if(p == 0)
	{
		return 1;
	}
	u32 rx_len = p[0]; //usually <= 255 so 1 byte should be sufficient
	if (rx_len)
	{
		tuya_ble_common_uart_receive_data(&p[4], rx_len);
		TUYA_BLE_LOG_HEXDUMP_INFO("UART RECV:", &p[4], rx_len);
	}

	my_fifo_pop(&spp_rx_fifo);
	return 0;
}


_attribute_no_ret_data_  uart_data_t   T_txdata_buf;
int tx_to_uart_cb (void)
{
	u8 *p = my_fifo_get (&spp_tx_fifo);
	if (p && !uart_tx_is_busy ())
	{
		memcpy(&T_txdata_buf.data, p + 2, p[0]+p[1]*256);
		T_txdata_buf.len = p[0]+p[1]*256 ;

		if (uart_dma_send((u8 *)(&T_txdata_buf)))
		{
			my_fifo_pop (&spp_tx_fifo);
		}
		TUYA_BLE_LOG_HEXDUMP_INFO("UART SEND:", &T_txdata_buf.data,T_txdata_buf.len);
	}
	return 0;
}


uint32_t ty_uart_init(void)
{
    ////////////////// SPP initialization ///////////////////////////////////
	//note: dma addr must be set first before any other uart initialization!
	u8 *uart_rx_addr = (spp_rx_fifo_b + (spp_rx_fifo.wptr & (spp_rx_fifo.num-1)) * spp_rx_fifo.size);
	uart_recbuff_init( (unsigned short *)uart_rx_addr, spp_rx_fifo.size);

	uart_gpio_set(TLSR_UART_GPIO_TX, TLSR_UART_GPIO_RX);

	uart_reset();  //will reset uart digital registers from 0x90 ~ 0x9f, so uart setting must set after this reset

	//baud rate: 115200
	#if (CLOCK_SYS_CLOCK_HZ == 16000000)
	    uart_init(TLSR_UART_BAUD_9600_P0, TLSR_UART_BAUD_9600_P1, PARITY_NONE, STOP_BIT_ONE);
	#elif (CLOCK_SYS_CLOCK_HZ == 24000000)
		uart_init(12, 15, PARITY_NONE, STOP_BIT_ONE);
	#elif (CLOCK_SYS_CLOCK_HZ == 32000000)
		uart_init(30,8,PARITY_NONE, STOP_BIT_ONE); //baud rate: 115200
	#elif (CLOCK_SYS_CLOCK_HZ == 48000000)
		uart_init(25,15,PARITY_NONE, STOP_BIT_ONE); //baud rate: 115200
	#endif

	uart_dma_enable(1, 1); 	//uart data in hardware buffer moved by dma, so we need enable them first

	irq_set_mask(FLD_IRQ_DMA_EN);
	dma_chn_irq_enable(FLD_DMA_CHN_UART_RX | FLD_DMA_CHN_UART_TX, 1);   	//uart Rx/Tx dma irq enable

	uart_irq_enable(0, 0);  	//uart Rx/Tx irq no need, disable them

    return 0;
}

uint32_t ty_uart2_init(void)
{
    return 0;
}

uint32_t ty_uart_send(const uint8_t* buf, uint32_t size)
{
	my_fifo_push(&spp_tx_fifo, buf, size);
    return 0;
}

uint32_t ty_uart2_send(const uint8_t* buf, uint32_t size)
{
    return 0;
}

uint32_t ty_uart_uninit(void)
{
    return 0;
}

uint32_t ty_uart2_uninit(void)
{
    return 0;
}

